<?php
namespace WSCOMUN\GVLOGIN;
use WSCOMUN\WSComunFClient;

/**
 * Librera para que aplicaciones PHP/GVHidra puedan acceder al Web Service de GVLogin
 *
 * @package		WSClientGVLogin
 * @author 		PAblo Belenguer <belenguer_pab@externosgva.es>
 * @author 		David Pascual <pascual_dav@externosgva.es>
 * @version		Release: @package_version@
 * @copyright 	Copyright (C) 2015 Generalitat Valenciana.
 * @license 	http://opensource.org/licenses/gpl-2.0.php GNU Public License v2.0
 * @link 		https://gvhidra.gva.es/redmine/projects/wscomun
 */
 class WSClientGVLogin extends WSComunFClient
{
	/**
	 * Es el mtodo que ofrece informacin disponible en el servicio referente al contexto de
	 * token de autenticacin/autorizacin: Dni, Nombre, Apellidos, Mail y Roles de autorizacin
	 * en las aplicaciones.
	 *
	 * @subpackage 	LOGIN
	 * @param  	string	$codApp	Cdigo del espacio de aplicacin invocante.
	 * @param  	string	$tokenSSO	Token identificativo del contexto
	 * @param  	string	$ip	Direccin ip del usuario que gener el token. Si es null, intenta autocompletarlo
	 * @param  	string	$agent	Cliente del usuario que gener el token. Si es null, intenta autocompletarlo
	 * @return	array 	Respuesta del servicio
	 * @access 	public
	 */
	public function obtenerContexto($codApp, $tokenSSO, $ip=null, $agent=null)
	{
		try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_GVLOGIN, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_GVLOGIN);


			// Valores por defecto
			if (($ip == null) || empty($ip))
			{
				$ip = $this->getAgentIP();
			}
			if (($agent == null) || empty($agent))
			{
				$agent = $this->getUserAgent();
			}


			// Compone la peticion
			$origenWS = new \ArrayObject();
			$origenWS->append(new \SoapVar($ip, XSD_STRING, null, null,'ip', self::$NAMESPACE_LOGIN));
			$origenWS->append(new \SoapVar($agent, XSD_STRING, null, null,'agent', self::$NAMESPACE_LOGIN));

			$mainWS = new \ArrayObject();
			$mainWS->append(new \SoapVar($codApp, XSD_STRING, null, null,'aplicacion', self::$NAMESPACE_LOGIN));
			$mainWS->append(new \SoapVar($tokenSSO, XSD_STRING, null, null,'tokenSSO', self::$NAMESPACE_LOGIN));
			$mainWS->append(new \SoapVar($origenWS, SOAP_ENC_OBJECT,null,null,'origen', self::$NAMESPACE_LOGIN));

			$miSoapRQ = new \SoapVar($mainWS, SOAP_ENC_OBJECT,null,null,'obtenerContexto', self::$NAMESPACE_LOGIN);
			$respuesta = $clienteWS->obtenerContexto($miSoapRQ);

			// Devuelve la respuesta, ya formateada en un objeto autonomo
			return new LoginResponse($respuesta);
		}
		catch (\Exception $e)
		{
			return $this->tratarInnerException($clienteWS, $e);
		}
	}//obtenerContexto



	/**
	 * Es el mtodo que comprueba la validez del token (SSO) y le aumenta la fecha
	 * de expiracin del token utilizado para el SSO.
	 *
	 * El incremento en la fecha de expiracin se basar en un parmetro de configuracin de GVLOGIN.
	 *
	 * @subpackage 	LOGIN
	 * @param  	string	$tokenSSO	Token identificativo del contexto
	 * @param  	string	$ip	Direccin ip del usuario que gener el token. Si es null, intenta autocompletarlo
	 * @param  	string	$agent	Cliente del usuario que gener el token. Si es null, intenta autocompletarlo
	 * @return	array 	Respuesta del servicio
	 * @access 	public
	 */
	public function verificarContexto($tokenSSO, $ip=null, $agent=null)
	{
			try
			{
				$opcionesClienteWS = array (
						'soap_version' => SOAP_1_1, //SOAP 1.2
						'user_agent' => 'WSSSoapClient',
						'exceptions' => true,
						'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
						'trace' => self::$MYSOAPOP_TRACE
				);

				$this->__clienteOn(self::KEYCLIENTE_GVLOGIN, true, $opcionesClienteWS);
				$clienteWS = $this->__getClient(self::KEYCLIENTE_GVLOGIN);


				// Valores por defecto
				if (($ip == null) || empty($ip))
				{
					$ip = $this->getAgentIP();
				}
				if (($agent == null) || empty($agent))
				{
					$agent = $this->getUserAgent();
				}


				// Compone la peticion
				$origenWS = new \ArrayObject();
				$origenWS->append(new \SoapVar($ip, XSD_STRING, null, null,'ip', self::$NAMESPACE_LOGIN));
				$origenWS->append(new \SoapVar($agent, XSD_STRING, null, null,'agent', self::$NAMESPACE_LOGIN));

				$mainWS = new \ArrayObject();
				$mainWS->append(new \SoapVar($tokenSSO, XSD_STRING, null, null,'tokenSSO', self::$NAMESPACE_LOGIN));
				$mainWS->append(new \SoapVar($origenWS, SOAP_ENC_OBJECT,null,null,'origen', self::$NAMESPACE_LOGIN));

				$miSoapRQ = new \SoapVar($mainWS, SOAP_ENC_OBJECT,null,null,'verificarContexto', self::$NAMESPACE_LOGIN);
				$respuesta = $clienteWS->verificarContexto($miSoapRQ);

				// Devuelve la respuesta, ya formateada en un objeto autonomo
				return new LoginResponse($respuesta);
			}
		catch (\Exception $e)
		{
			return $this->tratarInnerException($clienteWS, $e);
		}
	}//verificarContexto


	/**
	 * Es el mtodo que realiza el logout del token deshabilitando dicho token para el SSO.
	 *
	 * @subpackage 	LOGIN
	 * @param  	string	$tokenSSO	Token identificativo del contexto
	 * @return	array 	Respuesta del servicio
	 * @access 	public
	 */
	public function logout($tokenSSO)
	{
			try
		{
			$opcionesClienteWS = array (
					'soap_version' => SOAP_1_1, //SOAP 1.2
					'user_agent' => 'WSSSoapClient',
					'exceptions' => true,
					'cache_wsdl' => self::$MYSOAPOP_WSDL_CACHE,
					'trace' => self::$MYSOAPOP_TRACE
			);

			$this->__clienteOn(self::KEYCLIENTE_GVLOGIN, true, $opcionesClienteWS);
			$clienteWS = $this->__getClient(self::KEYCLIENTE_GVLOGIN);

			$mainWS = new \ArrayObject();
			$mainWS->append(new \SoapVar($tokenSSO, XSD_STRING, null, null,'tokenSSO', self::$NAMESPACE_LOGIN));

			$miSoapRQ = new \SoapVar($mainWS, SOAP_ENC_OBJECT,null,null,'logout', self::$NAMESPACE_LOGIN);
			$respuesta = $clienteWS->logout($miSoapRQ);

			// Devuelve la respuesta, ya formateada en un objeto autonomo
			return new LoginResponse($respuesta);
		}
		catch (\Exception $e)
		{
			return $this->tratarInnerException($clienteWS, $e);
		}
	}//logout




	/**
	 * Tratamiento estandarizado de las excepciones producidas en la invocacion al WS
	 *
	 * @param object	$clienteWS clienteWS utilizado en la peticion al Servicio Web
	 * @param object	$e Excepcion producida
	 */
	private function tratarInnerException($clienteWS, $e)
	{
		// Comprobamos si estamos ante excepcion MTOM
		$mensaje = $e->getMessage();
		$mensaje = trim(strtolower($mensaje));
		switch ($mensaje)
		{
			case 'looks like we got no xml document' :
				// Obtiene la respuesta
				$response = $clienteWS->__getLastResponse();
				$respuesta = $this->tratarMTOMEstandar($response);
				return $respuesta;
				break;

			default:
				$this->tratarExcepcionEstandar($e, $clienteWS);
				break;
		}
	}//tratarInnerException


	/**
	 * Devuelve la informacin del Agente (NAvegador) que ha accedido a GVLogin
	 * @access private
	 * @return string
	 */
	private function getUserAgent()
	{
		if (isset($_SERVER['HTTP_X_USER_AGENT']))
			return $_SERVER['HTTP_X_USER_AGENT'];
		return $_SERVER['HTTP_USER_AGENT'];
	}


	/**
	 *
	 * @return string
	 */
	private function getAgentIP()
	{
		$ip = '';
		if (getenv('HTTP_CLIENT_IP'))
		{
			$ip = getenv('HTTP_CLIENT_IP');
		}
		else if(getenv('HTTP_X_FORWARDED_FOR'))
		{
			$ip = getenv('HTTP_X_FORWARDED_FOR');
		}
		else if (getenv('HTTP_X_FORWARDED'))
		{
			$ip = getenv('HTTP_X_FORWARDED');
		}
		else if(getenv('HTTP_FORWARDED_FOR'))
		{
			$ip = getenv('HTTP_FORWARDED_FOR');
		}
		else if(getenv('HTTP_FORWARDED'))
		{
			$ip = getenv('HTTP_FORWARDED');
		}
		else if(getenv('REMOTE_ADDR'))
		{
			$ip = getenv('REMOTE_ADDR');
		}
		else
		{
			$ip = 'UNKNOWN';
		}
		return $ip;
	}//getAgentIP


	/**
	 *
	 * @return string
	 */
	private function getServerIP()
	{
		//return $_SERVER['REMOTE_ADDR'];
		$ip = $_SERVER['SERVER_ADDR'];

		if (in_array(PHP_OS, array('WINNT', 'WIN32', 'Windows')))
		{
			$ip = getHostByName(getHostName());
		}
		elseif (in_array(PHP_OS, array('Linux', 'Unix')))
		{
			$command="/sbin/ifconfig";
			$output = array();
			exec($command, $output);
			// var_dump($output);
			$pattern = '/inet addr:?([^ ]+)/';

			$ip = array();
			foreach ($output as $subject)
			{
				$subpattern = array();
				$result = preg_match_all($pattern, $subject, $subpattern);
				if ($result == 1)
				{
					if ($subpattern[1][0] != "127.0.0.1")
					{
						$ip = $subpattern[1][0];
					}
				}
				//var_dump($subpattern);
			}
		}
		return $ip;
	}//getServerIP


}//Class WSClientGVLogin

/* ----------------------------------------------------- */
/* --------- Clases de estructura del servicio --------- */
/* ----------------------------------------------------- */
class LoginResponse
{
	private $resultado = false;
	private $error = null;

	private $dni = null;
	private $nombre = null;
	private $apellido1 = null;
	private $apellido2 = null;
	private $mail = null;

	private $vParamInfoAmpliada = null;
	private $vRoles = null;

	/**
	 * Devuelve el resultado de la operacion
	 *
	 * @return	string	Respuesta del servicio
	 * @access 	public
	 */
	public function getResultado()
	{
		return $this->resultado;
	}


	/**
	 * Indica si ha ocurrido un error
	 *
	 * @return	boolean	Indica si ha habido un error
	 * @access 	public
	 */
	public function hasError()
	{
		return is_null($this->error);
	}


	/**
	 * Devuelve el error de la operacion, o null
	 *
	 * @return	object	Error del servicio (LoginError)
	 * @access 	public
	 */
	public function getError()
	{
		return $this->error;
	}


	/**
	 * Devuelve el campo 'dni' de la respuesta
	 *
	 * @return	String	Campo 'dni' de la respuesta
	 * @access 	public
	 */
	public function getDni()
	{
		return $this->dni;
	}

	/**
	 * Devuelve el campo 'nombre' de la respuesta
	 *
	 * @return	String	Campo 'nombre' de la respuesta
	 * @access 	public
	 */
	public function getNombre()
	{
		return $this->nombre;
	}

	/**
	 * Devuelve el campo 'apellido1' de la respuesta
	 *
	 * @return	String	Campo 'apellido1' de la respuesta
	 * @access 	public
	 */
	public function getApellido1()
	{
		return $this->apellido1;
	}

	/**
	 * Devuelve el campo 'apellido2' de la respuesta
	 *
	 * @return	String	Campo 'apellido2' de la respuesta
	 * @access 	public
	 */
	public function getApellido2()
	{
		return $this->apellido2;
	}

	/**
	 * Devuelve el campo 'mail' de la respuesta
	 *
	 * @return	String	Campo 'mail' de la respuesta
	 * @access 	public
	 */
	public function getMail()
	{
		return $this->mail;
	}


	/**
	 * Indica si hay parametros de informacion ampliada
	 *
	 * @return	boolean	Indica si hay parametros de informacion ampliada
	 * @access 	public
	 */
	public function hasInfoAmpliada()
	{
		return ($this->sizeOfInfoAmpliada() > 0);
	}


	/**
	 * Devuelve el numero de parametros de la informacion ampliada existente
	 *
	 * @return	int	Numero de parametros de la informacion ampliada
	 * @access 	public
	 */
	public function sizeOfInfoAmpliada()
	{
		if (is_null($this->vParamInfoAmpliada))
		{
			return 0;
		}
		return sizeof($this->vParamInfoAmpliada);
	}

	/**
	 * Devuelve el parametro de la informacion ampliada solicitado
	 *
	 * @param	int	$index Indice del elemento a devolver
	 * @return	LoginParametro	Elemento solicitado
	 * @access 	public
	 */
	public function getInfoAmpliada($index)
	{
		return $this->vParamInfoAmpliada[$index];
	}


	/**
	 * Indica si hay roles
	 *
	 * @return	boolean	Indica si hay roles
	 * @access 	public
	 */
	public function hasRoles()
	{
		return ($this->sizeOfRoles() > 0);
	}


	/**
	 * Devuelve el numero de roles existente
	 *
	 * @return	int	Numero de roles
	 * @access 	public
	 */
	public function sizeOfRoles()
	{
		if (is_null($this->vRoles))
		{
			return 0;
		}
		return sizeof($this->vRoles);
	}

	/**
	 * Devuelve el rol solicitado
	 *
	 * @param	int	$index Indice del elemento a devolver
	 * @return	LoginRol	Elemento solicitado
	 * @access 	public
	 */
	public function getRoles($index)
	{
		return $this->vRoles[$index];
	}



	/**
	 * (Constructor) Devuelve un BasicResponse a partir de una respuesta del servicio
	 *
	 * @return	object BasicResponse	Objecto creado
	 * @access 	public
	 */
	public function __construct($response)
	{
		$this->resultado = $response->resultado;

		if (($this->resultado == true) && isset($response->datos))
		{
			// Intenta capturar el resto de datos
			$datos = $response->datos;

			// Datos basicos
			if (isset($datos->dni))
			{
				$this->dni = $datos->dni;
			}
			if (isset($datos->nombre))
			{
				$this->nombre = $datos->nombre;
			}
			if (isset($datos->apellido1))
			{
				$this->apellido1 = $datos->apellido1;
			}
			if (isset($datos->apellido2))
			{
				$this->apellido2 = $datos->apellido2;
			}
			if (isset($datos->mail))
			{
				$this->mail = $datos->mail;
			}

			// InfoAmpliada
			if (isset($datos->infoAmpliada) && isset($datos->infoAmpliada->parametro))
			{
				// Normaliza asegurandose de que llega un array
				$vParamInfoAmpliada = $datos->infoAmpliada->parametro;
				if (!is_array($vParamInfoAmpliada)) $vParamInfoAmpliada = array($vParamInfoAmpliada);

				// Almacena los parametros
				for ($iParam=0; $iParam < sizeof($vParamInfoAmpliada); $iParam++)
				{
					$param = $vParamInfoAmpliada[$iParam];
					$this->vParamInfoAmpliada[] = new LoginParametro($param->nombreParametro, $param->valorParametro);
				}
			}

			// Roles
			if (isset($datos->roles) && isset($datos->roles->role))
			{
				// Normaliza asegurandose de que llega un array
				$vRoles = $datos->roles->role;
				if (!is_array($vRoles)) $vRoles = array($vRoles);

				// Almacena los roles y crea el array de roles
				$this->vRoles = array();
				for ($iRol=0; $iRol < sizeof($vRoles); $iRol++)
				{
					$this->vRoles[] = new LoginRol( $vRoles[$iRol] );
				}
			}
		}
		else
		{
			if (!empty($response->error))
			{
				$this->error = new BasicError($response->error);
			}
		}
	}
}

class LoginParametro
{

	private $nombre = null;
	private $valor = null;

	/**
	 * Devuelve el campo 'nombre' del parametro
	 *
	 * @return	String	Campo 'nombre' del parametro
	 * @access 	public
	 */
	public function getNombre() {
		return $this->nombre;
	}

	/**
	 * Devuelve el campo 'valor' del parametro
	 *
	 * @return	String	Campo 'valor' del parametro
	 * @access 	public
	 */
	public function getValor() {
		return $this->valor;
	}


	/**
	 * (Constructor) LoginParametro
	 *
	 * @param  	string	$nombre	Nombre del parametro
	 * @param  	string	$valor	Valor del parametro
	 */
	public function __construct($nombre, $valor) {
		$this->nombre = $nombre;
		$this->valor = $valor;
	}
}

class LoginRol
{

	private $codigo = null;
	private $vParametro = null;

	/**
	 * Devuelve el campo 'codigo' del parametro
	 *
	 * @return	String	Campo 'codigo' del parametro
	 * @access 	public
	 */
	public function getCodigo() {
		return $this->codigo;
	}

	/**
	 * Indica si hay parametros
	 *
	 * @return	boolean	Indica si hay parametros
	 * @access 	public
	 */
	public function hasParametro() {
		return ($this->sizeOfParametro() > 0);
	}


	/**
	 * Devuelve el numero de parametros existente
	 *
	 * @return	int	Numero de parametros
	 * @access 	public
	 */
	public function sizeOfParametro() {
		if (is_null($this->vParametro)) {
			return 0;
		}
		return sizeof($this->vParametro);
	}

	/**
	 * Devuelve el parametro solicitado
	 *
	 * @param	int	$index Indice del elemento a devolver
	 * @return	LoginParametro	Elemento solicitado
	 * @access 	public
	 */
	public function getParametro($index) {
		return $this->vParametro[$index];
	}


	/**
	 * (Constructor) Devuelve un LoginRole a partir de una respuesta del servicio
	 *
	 * @return	object LoginRole	Objecto creado
	 * @access 	public
	 */
	public function __construct($datos) {
		// Datos basicos
		if (isset($datos->codigo)) {
			$this->codigo = $datos->codigo;
		}

		// Parametros
		if (isset($datos->parametros) && isset($datos->parametros->parametro)) {
			// Normaliza asegurandose de que llega un array
			$vParam = $datos->parametros->parametro;
			if (!is_array($vParam)) $vParam = array($vParam);

			// Almacena los parametros y crea el array de parametros
			$this->vParametro = array();

			for ($iParam=0; $iParam < sizeof($vParam); $iParam++) {
				$param = $vParam[$iParam];
				$this->vParametro[] = new LoginParametro($param->nombreParametro, $param->valorParametro);
			}
		}
	}
}



class BasicError
{

/*
001 - Token no Existente - No existe el token.
002 - Aplicacion no Existente - La aplicacin no existe en el sistema.
003 - Token Caducado - El token ha caducado
004 - El usuario no existe en CLAU - No existe el usuario en el sistema CLAU
005 - El origen no coincide con el esperado - El origen no coincide con el esperado.
006 - Componente de postProcesamiento no encontrado - No se ha encontrado el componente para el tipo de postprocesamiento
007 - El usuario no esta activo en CLAU - El usuario no est activo en el sistema CLAU
008 - No se ha encontrado el componente para el tipo de autorizacion - No se ha encontrado el componente para el tipo de autorizacin
009 - El usuario no ha sido autorizado por Boperit - El usuario no ha sido autorizado por Boperit
010 - Error al invocar a los servicios de autorizacion - Error al invocar a los servicios de autorizacion.
011 - El usuario no tiene roles para la aplicacion - El usuario no tiene roles para la aplicacion
012 - Error al invocar a los servicios de postprocesamiento - Error al invocar a los servicios de postprocesamiento
013 - Aplicacion No Cumple Nivel Minimo Seguridad - La aplicacin no cumple el nivel mnimo de seguridad configurado para su acceso
GV-999 - Error Inesperado - Error genrico
*/

	const TOKEN_NO_EXISTENTE = '001';
	const APLICACION_NO_EXISTENTE = '002';
	const TOKEN_CADUCADO = '003';
	const USUARIO_NO_EXISTE_EN_CLAU = '004';
	const ORIGEN_NO_ESPERADO = '005';
	const COMPONENTE_PROSPROCESAMIENTO_NO_ENCONTRADO = '006';
	const USUARIO_NO_ACTIVO_EN_CLAU = '007';
	const COMPONENTE_AUTORIZACION_NO_ENCONTRADO = '008';
	const USUARIO_NO_AUTORIZADO_BOPERIT = '009';
	const ERROR_INVOCANDO_AUTORIZACION = '010';
	const USUARIO_SIN_ROLES = '011';
	const ERROR_INVOCANDO_POSTPROCESAMIENTO = '012';
	const APLICION_NO_CUMPLE_SEGURIDAD_MINIMA = '013';
	const ERROR_INESPERADO = '999';	// GV-999


	private $codigoError = '';
	private $mensajeError = '';


	public function getCodigoError()
	{
		return $this->codigoError;
	}
	public function getMensajeError()
	{
		return $this->mensajeError;
	}



	public function __construct($objError)
	{
		$this->codigoError = $objError->codigoError;
		$this->mensajeError = $objError->mensajeError;
	}

	public function __toString()
	{
		return __CLASS__ . ": [{$this->codigoError}] {$this->mensajeError}\n";
	}

}

?>